/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import java.util.ArrayList;
import java.util.List;

import aspmodificator.model.LineBase;
import aspmodificator.parser.gringo.GringoRule;
import aspmodificator.parser.gringo.Literal;

public class AddDefaultNegationModificationRule extends
		SwitchDefaultNegationModificationRule {

	public AddDefaultNegationModificationRule() {
		super();
	}

	public AddDefaultNegationModificationRule(int minApplication,
			int maxApplication) {
		super(minApplication, maxApplication);
	}

	@Override
	protected List<Literal> findApplicableObjects(List<LineBase> program) {
		List<Literal> result = new ArrayList<Literal>();

		for (LineBase line : program) {
			if (GringoRule.class.isAssignableFrom(line.getClass())) {
				GringoRule rule = (GringoRule) line;

				if (rule.getBody() != null) {
					for (Literal lit : rule.getBody()) {
						if (allowsDefaultNegation(lit.getClass())
								&& !lit.isModified() && !lit.isSigned()) {
							// test if resulting rule is safe
							lit.setSigned(true);

							if (GringoRuleUtil.checkSafety(rule))
								result.add(lit);

							lit.setSigned(false);
						}
					}
				}
			}
		}

		return result;
	}

	@Override
	protected void modifyLiteral(Literal lit) {
		lit.setSigned(true);

		if (writeComments) {
			lit.addSucceedingComment(COMMENT_ADDED);
		}
	}

}
