/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.regex.Pattern;

import common.Pair;
import common.Tripple;

import aspmodificator.model.LineBase;
import aspmodificator.parser.gringo.GringoRule;
import aspmodificator.parser.gringo.Literal;
import aspmodificator.parser.gringo.Term;
import aspmodificator.parser.gringo.VarTerm;

public class ConstantToVariableModificationRule extends
		AbstractReplacementLiteralModificationRule<String> {

	protected static final String COMMENT_CONSTANT_TO_VARIABLE = "%*ModificationNote: term has been modified from a constant to a variable*%";

	public ConstantToVariableModificationRule() {
		super();
	}

	public ConstantToVariableModificationRule(int minApplication,
			int maxApplication) {
		super(minApplication, maxApplication);
	}

	private boolean checkPreconditions(Literal lit) {
		Term term = (Term) lit;

		if (term.getTerms() != null && term.getTerms().size() > 0)
			return false;

		if (Pattern.matches("[a-z].*", term.getIdentifier()))
			return true;

		return false;
	}

	protected List<Tripple<Literal, Literal, List<String>>> findApplicableObjects(
			List<LineBase> program) {
		List<Tripple<Literal, Literal, List<String>>> result = new ArrayList<Tripple<Literal, Literal, List<String>>>();

		for (LineBase line : program) {
			if (GringoRule.class.isAssignableFrom(line.getClass())) {
				GringoRule rule = (GringoRule) line;

				// get all variables
				HashSet<String> variables = new HashSet<String>();
				List<VarTerm> tmp;
				if (rule.getHead() != null && !rule.getHead().isEmpty()) {
					for (Literal lit : rule.getHead()) {
						tmp = lit.getVariables();
						if (tmp != null && tmp.size() > 0) {
							for (Literal l2 : tmp) {
								variables.add(l2.getIdentifier());
							}
						}
					}
				}
				if (rule.getBody() != null && !rule.getBody().isEmpty()) {
					for (Literal lit : rule.getBody()) {
						tmp = lit.getVariables();
						if (tmp != null && tmp.size() > 0) {
							for (Literal l2 : tmp) {
								variables.add(l2.getIdentifier());
							}
						}
					}
				}
				List<String> variableNames = new ArrayList<String>(variables);

				if (rule.getHead() != null
						&& (mrRuleTarget == RuleTargetEnum.ALL || mrRuleTarget == RuleTargetEnum.HEAD)) {
					List<Pair<Literal, Literal>> termList = Literal
							.findClassesByTypeWithParent(Term.class,
									rule.getHead(), null);

					for (Pair<Literal, Literal> pair : termList) {
						if (checkPreconditions(pair.getFirst())
								&& !pair.getFirst().isModified())
							result.add(new Tripple<Literal, Literal, List<String>>(
									pair.getFirst(), pair.getSecond(),
									variableNames));
					}
				}

				if (rule.getBody() != null
						&& (mrRuleTarget == RuleTargetEnum.ALL || mrRuleTarget == RuleTargetEnum.BODY)) {
					List<Pair<Literal, Literal>> termList = Literal
							.findClassesByTypeWithParent(Term.class,
									rule.getBody(), null);

					for (Pair<Literal, Literal> pair : termList) {
						if (checkPreconditions(pair.getFirst())
								&& !pair.getFirst().isModified())
							result.add(new Tripple<Literal, Literal, List<String>>(
									pair.getFirst(), pair.getSecond(),
									variableNames));
					}
				}
			}

		}

		return result;
	}

	@Override
	public ModificationRuleTypeEnum getRuleTypeEnum() {
		return ModificationRuleTypeEnum.TCV;
	}

	protected Literal modifyLiteral(Pair<Literal, Literal> pair) {
		String name = pair.getFirst().getIdentifier().substring(0, 1)
				.toUpperCase()
				+ pair.getFirst().getIdentifier().substring(1);

		Literal newLit = new VarTerm(name, pair.getFirst().getLineNo());
		if (writeComments) {
			newLit.addSucceedingComment(COMMENT_CONSTANT_TO_VARIABLE);
		}

		return newLit;
	}

	@Override
	protected Literal modifyLiteral(
			Tripple<Literal, Literal, List<String>> tripple, int index) {

		Literal newLit = new VarTerm(tripple.getThird().get(index), tripple
				.getFirst().getLineNo());
		if (writeComments) {
			newLit.addSucceedingComment(COMMENT_CONSTANT_TO_VARIABLE);
		}

		return newLit;
	}

}
