/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import aspmodificator.model.LineBase;
import aspmodificator.parser.gringo.ConstTerm;
import aspmodificator.parser.gringo.GringoRule;
import aspmodificator.parser.gringo.Literal;
import aspmodificator.parser.gringo.Term;

import common.Pair;
import common.Tripple;

public class ReplaceConstantModificationRule extends
		AbstractReplacementLiteralModificationRule<String> {

	public ReplaceConstantModificationRule() {
		super();
	}

	public ReplaceConstantModificationRule(int minApplication,
			int maxApplication) {
		super(minApplication, maxApplication);
	}

	@Override
	protected List<Tripple<Literal, Literal, List<String>>> findApplicableObjects(
			List<LineBase> program) {
		List<Tripple<Literal, Literal, List<String>>> result = new ArrayList<Tripple<Literal, Literal, List<String>>>();
		HashSet<String> constNames = new HashSet<String>();
		HashSet<String> integerNames = new HashSet<String>();

		for (LineBase line : program) {
			if (GringoRule.class.isAssignableFrom(line.getClass())) {
				GringoRule rule = (GringoRule) line;

				if (rule.getHead() != null) {
					for (Literal lit : rule.getHead()) {
						if (lit.isModified())
							continue;

						// constants
						List<Literal> tmpList = Literal.findClassesByType(
								Term.class, lit);

						for (Literal mt : tmpList) {
							constNames.add(mt.getIdentifier());

							if (!mt.isModified()
									&& (mrRuleTarget == RuleTargetEnum.ALL || mrRuleTarget == RuleTargetEnum.HEAD))
								result.add(new Tripple<Literal, Literal, List<String>>(
										mt, null, new ArrayList<String>()));
						}

						// integers
						tmpList = Literal.findClassesByType(ConstTerm.class,
								lit);

						for (Literal mt : tmpList) {
							constNames.add(mt.getIdentifier());
							integerNames.add(mt.getIdentifier());

							if (!mt.isModified()
									&& (mrRuleTarget == RuleTargetEnum.ALL || mrRuleTarget == RuleTargetEnum.HEAD))
								result.add(new Tripple<Literal, Literal, List<String>>(
										mt, null, new ArrayList<String>()));
						}
					}
				}

				if (rule.getBody() != null) {
					for (Literal lit : rule.getBody()) {
						if (lit.isModified())
							continue;

						// find constants (lowercase)
						List<Literal> tmpList = Literal.findClassesByType(
								Term.class, lit);

						for (Literal mt : tmpList) {

							constNames.add(mt.getIdentifier());

							if (!mt.isModified()
									&& (mrRuleTarget == RuleTargetEnum.ALL || mrRuleTarget == RuleTargetEnum.BODY))
								result.add(new Tripple<Literal, Literal, List<String>>(
										mt, null, new ArrayList<String>()));
						}

						// find integers
						tmpList = Literal.findClassesByType(ConstTerm.class,
								lit);

						for (Literal mt : tmpList) {
							constNames.add(mt.getIdentifier());
							integerNames.add(mt.getIdentifier());

							if (!mt.isModified()
									&& (mrRuleTarget == RuleTargetEnum.ALL || mrRuleTarget == RuleTargetEnum.BODY))
								result.add(new Tripple<Literal, Literal, List<String>>(
										mt, null, new ArrayList<String>()));
						}
					}
				}
			}
		}

		List<Tripple<Literal, Literal, List<String>>> removeList = new ArrayList<Tripple<Literal, Literal, List<String>>>();

		for (Tripple<Literal, Literal, List<String>> pair : result) {

			// if the constant is an integer, it should only be replaced by
			// other integer values
			if (integerNames.contains(pair.getFirst().getIdentifier())) {
				integerNames.remove(pair.getFirst().getIdentifier());
				if (integerNames.isEmpty())
					removeList.add(pair);
				else
					pair.getThird().addAll(integerNames);
				integerNames.add(pair.getFirst().getIdentifier());
			} else {// constant is written in lowercase
				constNames.remove(pair.getFirst().getIdentifier());
				if (constNames.isEmpty())
					removeList.add(pair);
				else
					pair.getThird().addAll(constNames);
				constNames.add(pair.getFirst().getIdentifier());
			}
		}

		result.removeAll(removeList);

		return result;
	}

	@Override
	public ModificationRuleTypeEnum getRuleTypeEnum() {
		return ModificationRuleTypeEnum.TRC;
	}

	protected void modifyLiteral(Pair<Literal, List<String>> pair) {

	}

	@Override
	protected Literal modifyLiteral(
			Tripple<Literal, Literal, List<String>> tripple, int index) {
		Term constTerm = (Term) tripple.getFirst();

		if (writeComments) {
			constTerm
					.addSucceedingComment("%* ModificationNote: constant has been renamed. Original name was \""
							+ constTerm.getIdentifier() + "\" *%");
		}

		constTerm.setIdentifier(tripple.getThird().get(index));
		return constTerm;
	}
}
