/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import java.util.ArrayList;
import java.util.List;

import aspmodificator.model.LineBase;
import aspmodificator.parser.gringo.AggregatLiteralBracketsEnum;
import aspmodificator.parser.gringo.AggregateLiteral;
import aspmodificator.parser.gringo.CondLit;
import aspmodificator.parser.gringo.ConstTerm;
import aspmodificator.parser.gringo.GringoRule;
import aspmodificator.parser.gringo.Literal;
import aspmodificator.parser.gringo.MathTerm;

public class TwiddleAggregateWeightsModificationRule extends
		AbstractLiteralModificationRule {

	public static final String COMMENT_TWIDDL_WEIGHTS_INC = "%*ModificationNote: Weight increased by 1 *%";
	public static final String COMMENT_TWIDDL_WEIGHTS_DEC = "%*ModificationNote: Weight decreased by 1 *%";

	public TwiddleAggregateWeightsModificationRule() {
		super();
	}

	public TwiddleAggregateWeightsModificationRule(int minApplication,
			int maxApplication) {
		super(minApplication, maxApplication);
	}

	@Override
	protected List<Literal> findApplicableObjects(List<LineBase> program) {
		List<Literal> result = new ArrayList<Literal>();

		for (LineBase line : program) {
			if (GringoRule.class.isAssignableFrom(line.getClass())) {
				GringoRule rule = (GringoRule) line;

				if (rule.getHead() != null
						&& (mrRuleTarget == RuleTargetEnum.ALL || mrRuleTarget == RuleTargetEnum.HEAD)) {
					for (Literal lit : rule.getHead()) {
						if (lit.isModified())
							continue;

						if (AggregateLiteral.class.isAssignableFrom(lit
								.getClass())) {
							AggregateLiteral aggLit = (AggregateLiteral) lit;

							if (aggLit.getBrackets() == AggregatLiteralBracketsEnum.BRACETS_CURLY)
								continue;

							List<Literal> tmpList = Literal.findClassesByType(
									CondLit.class, lit);

							for (Literal mt : tmpList) {
								CondLit cl = (CondLit) mt;
								if (!mt.isModified() && !cl.hasDefaultWeight())
									result.add(mt);
							}
						}
					}
				}

				if (rule.getBody() != null
						&& (mrRuleTarget == RuleTargetEnum.ALL || mrRuleTarget == RuleTargetEnum.BODY)) {
					for (Literal lit : rule.getBody()) {
						if (lit.isModified())
							continue;
						if (AggregateLiteral.class.isAssignableFrom(lit
								.getClass())) {
							AggregateLiteral aggLit = (AggregateLiteral) lit;

							if (aggLit.getBrackets() == AggregatLiteralBracketsEnum.BRACETS_CURLY)
								continue;

							List<Literal> tmpList = Literal.findClassesByType(
									CondLit.class, lit);

							for (Literal mt : tmpList) {
								CondLit cl = (CondLit) mt;
								if (!mt.isModified() && !cl.hasDefaultWeight())
									result.add(mt);
							}
						}
					}
				}
			}
		}

		return result;
	}

	@Override
	public ModificationRuleTypeEnum getRuleTypeEnum() {
		return ModificationRuleTypeEnum.ATW;
	}

	@Override
	protected void modifyLiteral(Literal lit) {
		CondLit condLit = (CondLit) lit;

		boolean shouldIncrement = random.nextBoolean();

		if (ConstTerm.class.isAssignableFrom(condLit.getWeight().getClass())) {
			int value = Integer.valueOf(condLit.getWeight().getIdentifier());

			if (shouldIncrement || value == 1) {
				condLit.getWeight().setIdentifier(Integer.toString(value + 1));

				if (writeComments)
					condLit.getWeight().addSucceedingComment(
							COMMENT_TWIDDL_WEIGHTS_INC);
			} else {
				condLit.getWeight().setIdentifier(Integer.toString(value - 1));

				if (writeComments)
					condLit.getWeight().addSucceedingComment(
							COMMENT_TWIDDL_WEIGHTS_DEC);
			}
		} else {

			MathTerm mathTerm;

			if (shouldIncrement) {
				mathTerm = new MathTerm(MathTerm.PLUS, condLit.getWeight(),
						new ConstTerm(Integer.valueOf(1), lit.getLineNo()),
						lit.getLineNo());

				if (writeComments)
					condLit.getWeight().addSucceedingComment(
							COMMENT_TWIDDL_WEIGHTS_INC);
			} else {
				mathTerm = new MathTerm(MathTerm.MINUS, condLit.getWeight(),
						new ConstTerm(Integer.valueOf(1), lit.getLineNo()),
						lit.getLineNo());

				if (writeComments)
					condLit.getWeight().addSucceedingComment(
							COMMENT_TWIDDL_WEIGHTS_DEC);
			}

			condLit.setWeight(mathTerm);
		}

	}

}
