/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.parser.gringo;

import java.util.ArrayList;
import java.util.List;

public class AggregateLiteral extends Literal {

	public static final String MIN = "#min";
	public static final String PARITY_ODD = "PARITY_ODD";
	public static final String PARITY_EVEN = "PARITY_EVEN";
	public static final String SUM = "#sum";
	public static final String COUNT = "#count";

	private AggregatLiteralBracketsEnum brackets = null;

	private Term lower;

	private Term upper;

	public AggregateLiteral() {
		super();
		this.lower = null;
		this.upper = null;
	}

	/*
	 * public AggregateLiteral(String identifier, List<Literal> list, int
	 * lineNo) { this(identifier, list, false, lineNo); }
	 */
	public AggregateLiteral(String identifier, List<Literal> terms,
			AggregatLiteralBracketsEnum brackets, int lineNo) {
		super(identifier, terms, false);
		this.brackets = brackets;
	}

	/*
	 * public AggregateLiteral(String identifier, List<Literal> list) {
	 * this(identifier, list, false); }
	 * 
	 * public AggregateLiteral(String operation, List<Literal> terms, boolean
	 * signed) { super(operation, terms, signed); }
	 * 
	 * public AggregateLiteral(String operation, List<Literal> terms, boolean
	 * signed, int lineNo) { super(operation, terms, signed, lineNo); }
	 */
	public AggregateLiteral(String operation, List<Literal> terms,
			boolean signed, AggregatLiteralBracketsEnum brackets, int lineNo) {
		super(operation, terms, signed, lineNo);
		this.brackets = brackets;
	}

	@Override
	public Object clone() throws CloneNotSupportedException {
		List<Literal> newTerms = null;

		if (terms != null) {
			newTerms = new ArrayList<Literal>();
			for (Literal lit : terms) {
				newTerms.add((Literal) lit.clone());
			}
		}

		List<BlockComment> newPrecedingComments = null;

		if (precedingComments != null) {
			newPrecedingComments = new ArrayList<BlockComment>();
			for (BlockComment c : precedingComments) {
				newPrecedingComments.add((BlockComment) c.clone());
			}
		}

		List<BlockComment> newSucceedingComments = null;
		if (succeedingComments != null) {
			newSucceedingComments = new ArrayList<BlockComment>();
			for (BlockComment c : succeedingComments) {
				newSucceedingComments.add((BlockComment) c.clone());
			}
		}
		String newIdentifier = null;
		if (identifier != null)
			newIdentifier = new String(identifier);

		Term newLower = null;
		if (lower != null)
			newLower = (Term) lower.clone();

		Term newUpper = null;
		if (upper != null)
			newUpper = (Term) upper.clone();

		AggregateLiteral newLit = new AggregateLiteral(newIdentifier, newTerms,
				signed, brackets, lineNo);

		newLit.setLower(newLower);
		newLit.setUpper(newUpper);

		newLit.setPrecedingComments(newPrecedingComments);
		newLit.setSucceedingComments(newSucceedingComments);
		return newLit;
	}

	public AggregatLiteralBracketsEnum getBrackets() {
		return brackets;
	}

	public Term getLower() {
		return lower;
	}

	public Term getUpper() {
		return upper;
	}

	public void setLower(Term l) {
		this.lower = l;
	}

	public void setUpper(Term u) {
		this.upper = u;
	}

	@Override
	public String toString() {
		StringBuffer sb = new StringBuffer();

		if (lower != null) {
			sb.append(lower.toString());
			sb.append(" ");
		}

		if (identifier != null) {
			sb.append(identifier);
			sb.append(" ");
		}

		if (brackets == AggregatLiteralBracketsEnum.BRACETS_CURLY)
			sb.append("{ ");
		else if (brackets == AggregatLiteralBracketsEnum.BRACETS_SQUARE)
			sb.append("[ ");

		if (terms != null && terms.size() > 0) {
			for (int i = 0; i < terms.size() - 1; i++) {
				sb.append(terms.get(i).toString());
				sb.append(", ");
			}

			sb.append(terms.get(terms.size() - 1).toString());
		}

		if (brackets == AggregatLiteralBracketsEnum.BRACETS_CURLY)
			sb.append(" }");
		else if (brackets == AggregatLiteralBracketsEnum.BRACETS_SQUARE)
			sb.append(" ]");

		if (upper != null) {
			sb.append(" ");
			sb.append(upper.toString());
		}
		return sb.toString();
	}

}
