/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.parser.gringo;

import java.util.ArrayList;
import java.util.List;

public class CommentableObject implements ICommentable {

	private List<BlockComment> precedingComments = null;

	private List<BlockComment> succeedingComments = null;

	public CommentableObject() {
		super();
	}

	public CommentableObject(List<BlockComment> precedingComments,
			List<BlockComment> succeedingComments) {
		super();
		this.precedingComments = precedingComments;
		this.succeedingComments = succeedingComments;
	}

	@Override
	public void addPrecedingComment(String bc) {
		if (this.precedingComments == null)
			this.precedingComments = new ArrayList<BlockComment>();
		precedingComments.add(0, new BlockComment(bc));

	}

	@Override
	public void addSucceedingComment(String bc) {
		if (this.succeedingComments == null)
			this.succeedingComments = new ArrayList<BlockComment>();

		succeedingComments.add(0, new BlockComment(bc));
	}

	@Override
	protected Object clone() throws CloneNotSupportedException {
		List<BlockComment> newPrecedingComments = null;
		List<BlockComment> newSucceedingComments = null;

		if (precedingComments != null && precedingComments.size() > 0) {
			newPrecedingComments = new ArrayList<BlockComment>();
			for (BlockComment bc : precedingComments) {
				newPrecedingComments.add((BlockComment) bc.clone());
			}
		}

		if (succeedingComments != null && succeedingComments.size() > 0) {
			newSucceedingComments = new ArrayList<BlockComment>();
			for (BlockComment bc : succeedingComments) {
				newSucceedingComments.add((BlockComment) bc.clone());
			}
		}

		return new CommentableObject(newPrecedingComments,
				newSucceedingComments);
	}

	public List<BlockComment> getPrecedingComments() {
		return precedingComments;
	}

	public List<BlockComment> getSucceedingComments() {
		return succeedingComments;
	}

}
