/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.parser.gringo;

import java.util.ArrayList;
import java.util.List;

import aspmodificator.model.LineBase;

public class ComputeCommand extends LineBase {
	protected Integer number = null;
	protected List<Literal> computeLiteralList;

	public ComputeCommand(List<Literal> computeLiteralList, int startLineNo) {
		this(computeLiteralList, null, startLineNo);
	}

	public ComputeCommand(List<Literal> computeLiteralList, Integer num,
			int startLineNo) {
		super(startLineNo);
		this.computeLiteralList = computeLiteralList;
		this.number = num;
	}

	@Override
	public Object clone() throws CloneNotSupportedException {
		List<Literal> newList = null;

		if (computeLiteralList != null && computeLiteralList.size() > 0) {
			newList = new ArrayList<Literal>();
			for (Literal lit : computeLiteralList) {
				newList.add((Literal) lit.clone());
			}
		}

		return new ComputeCommand(newList, number, startLineNo);
	}

	public List<Literal> getComputeLiteralList() {
		return computeLiteralList;
	}

	public Integer getNumber() {
		return number;
	}

	public void setComputeLiteralList(List<Literal> computeLiteralList) {
		this.computeLiteralList = computeLiteralList;
	}

	public void setNumber(Integer number) {
		this.number = number;
	}

	@Override
	public String toOutputString() {
		int currentLineNo = startLineNo;

		StringBuffer sb = new StringBuffer();
		sb.append("#compute");

		if (number != null) {
			sb.append(" ");
			sb.append(number.toString());
		}

		sb.append(" { ");
		if (computeLiteralList != null && computeLiteralList.size() > 0) {
			Literal lit = null;
			for (int i = 0; i < computeLiteralList.size() - 1; i++) {
				lit = computeLiteralList.get(i);

				while (lit.getLineNo() > currentLineNo) {
					sb.append("\n");
					currentLineNo++;
				}

				sb.append(lit.toString());
				sb.append(", ");
			}

			lit = computeLiteralList.get(computeLiteralList.size() - 1);

			while (lit.getLineNo() > currentLineNo) {
				sb.append("\n");
				currentLineNo++;
			}

			sb.append(lit.toString());
		}
		sb.append(" }.");

		return sb.toString();
	}

	@Override
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("#compute");

		if (number != null) {
			sb.append(" ");
			sb.append(number.toString());
		}

		sb.append(" { ");
		if (computeLiteralList != null && computeLiteralList.size() > 0) {
			Literal lit = null;
			for (int i = 0; i < computeLiteralList.size() - 1; i++) {
				lit = computeLiteralList.get(i);
				sb.append(lit.toString());
				sb.append(", ");
			}

			lit = computeLiteralList.get(computeLiteralList.size() - 1);
			sb.append(lit.toString());
		}
		sb.append(" }.");

		return sb.toString();
	}

}
