/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.parser.gringo;

import java.util.ArrayList;
import java.util.List;

import aspmodificator.model.LineBase;

public class MetaStatement extends LineBase {
	public static final String SHOW = "#show";
	public static final String HIDE = "#hide";
	protected String token;
	private Predicate predicate;
	private List<Literal> literalList;
	private Signed id;
	private Integer number;

	public MetaStatement(String s) {
		this.token = s;
	}

	public MetaStatement(String s, int lineNo) {
		this.token = s;
		this.startLineNo = lineNo;
	}

	public MetaStatement(String tok, Predicate pred, List<Literal> list) {
		this.token = tok;
		this.predicate = pred;
		this.literalList = list;
	}

	public MetaStatement(String tok, Predicate pred, List<Literal> list,
			int lineNo) {
		this.token = tok;
		this.predicate = pred;
		this.literalList = list;
		this.startLineNo = lineNo;
	}

	public MetaStatement(String tok, Signed id, Integer num) {
		this.token = tok;
		this.id = id;
		this.number = num;
	}

	public MetaStatement(String tok, Signed id, Integer num, int lineNo) {
		this.token = tok;
		this.id = id;
		this.number = num;
		this.startLineNo = lineNo;
	}

	@Override
	public Object clone() throws CloneNotSupportedException {
		if (id != null) {
			return new MetaStatement(token, id, number);
		} else if (predicate != null) {
			List<Literal> newList = null;
			if (literalList != null) {
				newList = new ArrayList<Literal>();
				for (Literal lit : literalList) {
					newList.add((Literal) lit.clone());
				}
			}
			return new MetaStatement(token, (Predicate) predicate.clone(),
					newList, startLineNo);
		} else
			return new MetaStatement(token, startLineNo);
	}

	@Override
	public String toOutputString() {
		return toString();
	}

	@Override
	public String toString() {
		if (id != null) {
			return token + " " + id.toString() + "/" + number.toString() + ".";
		} else if (predicate != null) {
			StringBuffer sb = new StringBuffer();
			sb.append(token);
			sb.append(" ");
			sb.append(predicate.toString());

			if (literalList != null && literalList.size() > 0) {
				for (int i = 0; i < literalList.size() - 1; i++) {
					sb.append(literalList.get(i).toString());
					sb.append(", ");
				}

				sb.append(literalList.get(literalList.size() - 1).toString());
			}
			sb.append(".");
			return sb.toString();
		} else
			return token + ".";
	}

}
