/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.parser.gringo;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import aspmodificator.model.LineBase;

public class OptimizeCommand extends LineBase {

	private List<OptimizeLiteral> literals;
	private Optimize type;

	public OptimizeCommand(Optimize sopt, List<OptimizeLiteral> list) {
		this.type = sopt;
		this.literals = list;
	}

	public OptimizeCommand(Optimize sopt, List<OptimizeLiteral> list,
			int startLineNo) {
		this.type = sopt;
		this.literals = list;
		this.startLineNo = startLineNo;
	}

	public OptimizeCommand(Optimize sopt, Set<OptimizeLiteral> set) {
		this.type = sopt;
		this.literals = new ArrayList<OptimizeLiteral>(set);
	}

	public OptimizeCommand(Optimize sopt, Set<OptimizeLiteral> set,
			int startLineNo) {
		this.type = sopt;
		this.literals = new ArrayList<OptimizeLiteral>(set);
		this.startLineNo = startLineNo;
	}

	@Override
	public Object clone() throws CloneNotSupportedException {
		List<OptimizeLiteral> tmpList = new ArrayList<OptimizeLiteral>();

		if (literals != null && literals.size() > 0) {
			for (OptimizeLiteral lit : literals) {
				tmpList.add((OptimizeLiteral) lit.clone());
			}
		}
		return new OptimizeCommand(type, tmpList, startLineNo);
	}

	public List<OptimizeLiteral> getLiterals() {
		return literals;
	}

	public Optimize getType() {
		return type;
	}

	public void setLiterals(List<OptimizeLiteral> literals) {
		this.literals = literals;
	}

	public void setType(Optimize type) {
		this.type = type;
	}

	@Override
	public String toOutputString() {
		StringBuffer sb = new StringBuffer();
		sb.append(type.toString().toLowerCase());
		sb.append(" [ ");

		int currentLineNo = 0;
		OptimizeLiteral lit = null;

		if (literals != null) {
			// initialize currentLineNo
			currentLineNo = literals.get(0).getLineNo();

			for (int i = 0; i < literals.size() - 1; i++) {
				lit = literals.get(i);
				while (lit.getLineNo() > currentLineNo) {
					sb.append("\n");
					currentLineNo++;
				}
				sb.append(lit.toString());
				sb.append(",");
			}
			// write last element
			lit = literals.get(literals.size() - 1);
			while (lit.getLineNo() > currentLineNo) {
				sb.append("\n");
				currentLineNo++;
			}
			sb.append(lit.toString());
		}
		sb.append(" ].");
		return sb.toString();
	}

	@Override
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append(type.toString().toLowerCase());
		sb.append(" [ ");

		if (literals != null) {
			for (int i = 0; i < literals.size() - 1; i++) {
				sb.append(literals.get(i).toString());
				sb.append(",");
			}

			sb.append(literals.get(literals.size() - 1).toString());
		}
		sb.append(" ].");
		return sb.toString();
	}

}
