/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package main;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DecimalFormat;
import java.util.List;

import org.apache.log4j.Logger;

import aspmodificator.base.ProgramModificator;
import aspmodificator.model.LineBase;
import aspmodificator.parser.GringoParser;
import aspmodificator.parser.IFileParser;

public class AspModificator {
	private static Logger logger = Logger.getLogger(AspModificator.class);
	private ProgramModificator programModificator;
	private AspModificationConfig config;

	private IFileParser parser;

	public AspModificator() {
		programModificator = new ProgramModificator();
		parser = new GringoParser();
	}

	public void doYourJob() throws Exception {

		// do config
		programModificator.setWriteComments(config.isWriteComments());
		programModificator.setRandomInitialization(config.getRandomInit());
		programModificator.setRuleApplication(config.getRuleApplication());
		programModificator.setUseMemory(config.isUseMemory());

		// security checks
		if (config.getInputFile() == null
				|| config.getInputFile().isDirectory()
				|| !config.getInputFile().exists())
			throw new Exception("Invalid input-file");

		if (config.getOutputDir() == null) {
			config.setOutputDir(new File("."));
		}

		if (config.getOutputFileName() == null) {
			config.setOutputFileName(config
					.getInputFile()
					.getName()
					.substring(0,
							config.getInputFile().getName().lastIndexOf("."))
					+ "_mod"
					+ config.getInputFile()
							.getName()
							.substring(
									config.getInputFile().getName()
											.lastIndexOf(".")));
		}

		// read input file
		StringBuffer fileData = new StringBuffer(1000);
		BufferedReader reader = new BufferedReader(new FileReader(
				config.getInputFile()));
		char[] buf = new char[1024];
		int numRead = 0;
		while ((numRead = reader.read(buf)) != -1) {
			String readData = String.valueOf(buf, 0, numRead);
			fileData.append(readData);
			buf = new char[1024];
		}
		reader.close();

		// parse Program
		List<LineBase> program = parser.parse(fileData.toString());

		// decimalFormat used to make of 1 -> 001
		int noZeros = (int) Math.log10(config.getNoCopies()) + 1;
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < noZeros; i++) {
			sb.append("0");
		}
		DecimalFormat df = new DecimalFormat(sb.toString());

		// apply modification Rules
		for (int i = 0; i < config.getNoCopies(); i++) {

			List<LineBase> modifiedProgram = programModificator.modifyProgram(
					program, config.getModificationRules());

			if (modifiedProgram == null) {
				logger.info("Unable to create additional mutants - only " + i
						+ " of " + config.getNoCopies() + " mutants created.");
				return;
			}

			// write output-program
			String newName = config.getOutputFileName().substring(0,
					config.getOutputFileName().lastIndexOf("."))
					+ df.format(i)
					+ config.getOutputFileName().substring(
							config.getOutputFileName().lastIndexOf("."));

			writeProgramToFile(modifiedProgram, new File(config.getOutputDir()
					.toString() + File.separator + newName));

		}
	}

	public void setConfig(AspModificationConfig config) {
		this.config = config;
	}

	public void setInputFile(File inputFile) {
		config.setInputFile(inputFile);
	}

	public void setNoCopies(int value) {
		config.setNoCopies(value);
	}

	public void setOutputDir(File outputDir) {
		config.setOutputDir(outputDir);
	}

	public void setRandomInitialization(long seed) {
		config.setRandomInit(seed);
	}

	public void setWriteComments(boolean writeComments) {
		config.setWriteComments(writeComments);
	}

	private void writeProgramToFile(List<LineBase> modifiedProgram, File file)
			throws IOException {

		FileWriter out = new FileWriter(file);
		int outputLineNo = 1;
		int originalLineNo = 1;
		for (LineBase currentLine : modifiedProgram) {

			originalLineNo = currentLine.getStartLineNo();

			while (originalLineNo > outputLineNo) {
				out.write('\n');
				outputLineNo++;
			}

			// write line
			String tmpString = currentLine.toOutputString();
			out.write(tmpString);

			int noNewLine = tmpString.replaceAll("[^\n]", "").length();

			outputLineNo += noNewLine;
		}

		out.close();
	}

}
