/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import static org.junit.Assert.*;
import static org.hamcrest.core.Is.is;

import java.util.List;

import org.junit.Before;
import org.junit.Test;

import aspmodificator.model.LineBase;
import aspmodificator.parser.GringoParser;
import aspmodificator.parser.gringo.GringoRule;

public class GringoRuleUtilTest {
	private GringoParser parser;

	@Before
	public void setUp() throws Exception {
		parser = new GringoParser();
	}

	@Test
	public void testCheckSafety_shouldReturnFalse_1() throws Exception {
		String code = ":- not b(X).";
		List<LineBase> program = parser.parse(code);

		GringoRule rule = (GringoRule) program.get(0);

		boolean res = GringoRuleUtil.checkSafety(rule);
		assertThat(res, is(false));
	}

	@Test
	public void testCheckSafety_shouldReturnFalse_2() throws Exception {
		String code = "a(X) :- not b(X).";
		List<LineBase> program = parser.parse(code);

		GringoRule rule = (GringoRule) program.get(0);

		boolean res = GringoRuleUtil.checkSafety(rule);
		assertThat(res, is(false));
	}

	@Test
	public void testCheckSafety_shouldReturnFalse_3() throws Exception {
		String code = "a(X) :- b(X), not b(Y).";
		List<LineBase> program = parser.parse(code);

		GringoRule rule = (GringoRule) program.get(0);

		boolean res = GringoRuleUtil.checkSafety(rule);
		assertThat(res, is(false));
	}

	@Test
	public void testCheckSafety_shouldReturnFalse_4() throws Exception {
		String code = "a(X) :- b(Y).";
		List<LineBase> program = parser.parse(code);

		GringoRule rule = (GringoRule) program.get(0);

		boolean res = GringoRuleUtil.checkSafety(rule);
		assertThat(res, is(false));
	}

	@Test
	public void testCheckSafety_shouldReturnFalse_5() throws Exception {
		String code = "a(X).";
		List<LineBase> program = parser.parse(code);

		GringoRule rule = (GringoRule) program.get(0);

		boolean res = GringoRuleUtil.checkSafety(rule);
		assertThat(res, is(false));
	}

	@Test
	public void testCheckSafety_shouldReturnFalse_6() throws Exception {
		String code = "p(X) :- X < Y, q(Y).";
		List<LineBase> program = parser.parse(code);

		GringoRule rule = (GringoRule) program.get(0);

		boolean res = GringoRuleUtil.checkSafety(rule);
		assertThat(res, is(false));
	}

	@Test
	public void testCheckSafety_shouldReturnFalse_7() throws Exception {
		String code = "p(X) :- X == Y +1, q(Y).";
		List<LineBase> program = parser.parse(code);

		GringoRule rule = (GringoRule) program.get(0);

		boolean res = GringoRuleUtil.checkSafety(rule);
		assertThat(res, is(false));
	}

	@Test
	public void testCheckSafety_shouldReturnFalse_8() throws Exception {
		String code = "a(X) :- b, X = X * 1.";
		List<LineBase> program = parser.parse(code);

		GringoRule rule = (GringoRule) program.get(0);

		boolean res = GringoRuleUtil.checkSafety(rule);
		assertThat(res, is(false));
	}

	@Test
	public void testCheckSafety_shouldReturnTrue_1() throws Exception {
		String code = "a(X) :- b(X), c(X).";
		List<LineBase> program = parser.parse(code);

		GringoRule rule = (GringoRule) program.get(0);

		boolean res = GringoRuleUtil.checkSafety(rule);
		assertTrue(res);
	}

	@Test
	public void testCheckSafety_shouldReturnTrue_10() throws Exception {
		String code = ":- a(X), X = 1.";
		List<LineBase> program = parser.parse(code);

		GringoRule rule = (GringoRule) program.get(0);

		boolean res = GringoRuleUtil.checkSafety(rule);
		assertTrue(res);
	}

	@Test
	public void testCheckSafety_shouldReturnTrue_11() throws Exception {
		String code = ":- a(X).";
		List<LineBase> program = parser.parse(code);

		GringoRule rule = (GringoRule) program.get(0);

		boolean res = GringoRuleUtil.checkSafety(rule);
		assertTrue(res);
	}

	@Test
	public void testCheckSafety_shouldReturnTrue_12() throws Exception {
		String code = "p(X) :- X = Y +1, q(Y).";
		List<LineBase> program = parser.parse(code);

		GringoRule rule = (GringoRule) program.get(0);

		boolean res = GringoRuleUtil.checkSafety(rule);
		assertThat(res, is(true));
	}

	@Test
	public void testCheckSafety_shouldReturnTrue_13() throws Exception {
		String code = "p(X) :- X := Y +1, q(Y).";
		List<LineBase> program = parser.parse(code);

		GringoRule rule = (GringoRule) program.get(0);

		boolean res = GringoRuleUtil.checkSafety(rule);
		assertThat(res, is(true));
	}

	@Test
	public void testCheckSafety_shouldReturnTrue_14() throws Exception {
		String code = "p(X+1) :- q(X) .";
		List<LineBase> program = parser.parse(code);

		GringoRule rule = (GringoRule) program.get(0);

		boolean res = GringoRuleUtil.checkSafety(rule);
		assertThat(res, is(true));
	}

	@Test
	public void testCheckSafety_shouldReturnTrue_2() throws Exception {
		String code = "a(X) :- b(X), not c(X).";
		List<LineBase> program = parser.parse(code);

		GringoRule rule = (GringoRule) program.get(0);

		boolean res = GringoRuleUtil.checkSafety(rule);
		assertTrue(res);
	}

	@Test
	public void testCheckSafety_shouldReturnTrue_3() throws Exception {
		String code = "a(x).";
		List<LineBase> program = parser.parse(code);

		GringoRule rule = (GringoRule) program.get(0);

		boolean res = GringoRuleUtil.checkSafety(rule);
		assertTrue(res);
	}

	@Test
	public void testCheckSafety_shouldReturnTrue_4() throws Exception {
		String code = ":- a(0).";
		List<LineBase> program = parser.parse(code);

		GringoRule rule = (GringoRule) program.get(0);

		boolean res = GringoRuleUtil.checkSafety(rule);
		assertTrue(res);
	}

	@Test
	public void testCheckSafety_shouldReturnTrue_5() throws Exception {
		String code = ":- a(X).";
		List<LineBase> program = parser.parse(code);

		GringoRule rule = (GringoRule) program.get(0);

		boolean res = GringoRuleUtil.checkSafety(rule);
		assertTrue(res);
	}

	@Test
	public void testCheckSafety_shouldReturnTrue_6() throws Exception {
		String code = ":- a(X), not b(X).";
		List<LineBase> program = parser.parse(code);

		GringoRule rule = (GringoRule) program.get(0);

		boolean res = GringoRuleUtil.checkSafety(rule);
		assertTrue(res);
	}

	@Test
	public void testCheckSafety_shouldReturnTrue_7() throws Exception {
		String code = "a(1) :- not a(0).";
		List<LineBase> program = parser.parse(code);

		GringoRule rule = (GringoRule) program.get(0);

		boolean res = GringoRuleUtil.checkSafety(rule);
		assertTrue(res);
	}

	@Test
	public void testCheckSafety_shouldReturnTrue_8() throws Exception {
		String code = "a(x).";
		List<LineBase> program = parser.parse(code);

		GringoRule rule = (GringoRule) program.get(0);

		boolean res = GringoRuleUtil.checkSafety(rule);
		assertTrue(res);
	}

	@Test
	public void testCheckSafety_shouldReturnTrue_9() throws Exception {
		String code = "a(1):- not a(X), X = 1.";
		List<LineBase> program = parser.parse(code);

		GringoRule rule = (GringoRule) program.get(0);

		boolean res = GringoRuleUtil.checkSafety(rule);
		assertTrue(res);
	}

}
